(function () {
    var version = '1.0.0';
    var urlPrefix = 'js://invoke?';

    var isAndroid = (function () {
        return navigator.userAgent.match(/android/i);
    })();

    var WeixinJSBridge = function () {
        this.serialNo = 0;
        this.callbacks = {};
        this.events = {};
    };

    WeixinJSBridge.prototype.version = version;

    WeixinJSBridge.prototype.on = function (type, cb) {
        this.events[type] = cb;
    };

    WeixinJSBridge.prototype.onInvokeResult = function (serailNo, param) {
        var cb = this.callbacks[serailNo];
        if (!!!cb) {
            window.console.log("non-existed serail: " + serailNo);
            return false;
        }
        delete this.callbacks[serailNo];
        cb(param);
        return true;
    };

    WeixinJSBridge.prototype.onInvokeEvent = function (type, param) {
        var cb = this.events[type];
        if (!!!cb) {
            window.console.log("non-existed type: " + type);
            return false;
        }
        cb(param);
        return true;
    };

    WeixinJSBridge.prototype.invoke = function (name, param, cb, cancel) {
        ++this.serialNo;
        var serialNo = this.serialNo;
        if (!!cb) {
            this.callbacks[serialNo] = cb;
        }
        if (isAndroid) {
            this.invokeOnAndroid(name, param, this.serialNo);
        } else {
            this.invokeOnIos(name, param, this.serialNo);
        }
    };


    WeixinJSBridge.prototype.invokeOnAndroid = function (name, param, serialNo) {
        var obj = { "name": name, "params": param, "version": version, "serial": serialNo };
        var command = encodeURIComponent(JSON.stringify(obj));
        if (command) {
            window.prompt(urlPrefix + command);
        }
    };

    WeixinJSBridge.prototype.invokeOnIos = function (name, param, serialNo) {
        var location = urlPrefix + 'version=' + version + '&name=' + encodeURIComponent(name) + '&param=' + encodeURIComponent(JSON.stringify(param)) + '&serial=' + serialNo;
        window.console.log(location);
        window.location = location;
    };

    if (!!!window.WeixinJSBridge) {
        var WeixinJSBridge = new WeixinJSBridge;
        window.WeixinJSBridge = WeixinJSBridge;
        var ev = document.createEvent('Event');
        ev.initEvent("WeixinJSBridgeReady", true, true);
        document.dispatchEvent(ev);
    } else {
        console.log("WeixinJSBridge was already exist!!!")
    }
})();

